@@ -650,6 +650,22 @@ static int encode16_data(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	return 40 * 8;
 }
 
+/* TS 48.060 section 5.5.3 */
+static int encode16_d145_sync(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
+{
+	encode_sync16(trau_bits);
+
+	/* C1 .. C5 */
+	memcpy(trau_bits + 17, ft_d145s_bits, 5);
+	/* C6 .. C15 */
+	memcpy(trau_bits + 17 + 5, fr->c_bits + 5, 15 - 5);
+
+	/* This frame is for sync only, all data bits filled with 1s */
+	memset(trau_bits + 4 * 8, 1, (40 - 4) * 8);
+
+	return 40 * 8;
+}
+
 /* TS 08.60 3.3.2 */
 static int decode16_edata(struct osmo_trau_frame *fr, const ubit_t *trau_bits,
 			  enum osmo_trau_frame_direction dir)
@@ -676,6 +692,10 @@ static int encode16_edata(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	else
 		cbits5 = ft_edata_bits;
 
+	/* sync pattern */
+	memset(trau_bits, 0, 16);
+	trau_bits[16] = 1;
+
 	/* C1 .. C5 */
 	memcpy(trau_bits + 17, cbits5, 5);
 
@@ -780,15 +800,14 @@ static int decode8_hr(struct osmo_trau_frame *fr, const ubit_t *trau_bits,
 	}
 
 	/* CRC0 .. CRC2 */
-	fr->crc_bits[2] = trau_bits[82];
-	fr->crc_bits[1] = trau_bits[83];
-	fr->crc_bits[0] = trau_bits[84];
+	memcpy(fr->crc_bits, trau_bits + 73, 3);
 
 	/* D45 .. D48 */
-	memcpy(fr->d_bits + d_idx, trau_bits + 85, 4);
+	memcpy(fr->d_bits + d_idx, trau_bits + 76, 4);
+	d_idx += 4;
 
 	/* D49 .. D111 */
-	for (i = 10; i < 10 + 10; i++) {
+	for (i = 10; i < 10 + 9; i++) {
 		int offset = i * 8;
 		memcpy(fr->d_bits + d_idx, trau_bits + offset + 1, 7);
 		d_idx += 7;
@@ -843,7 +862,7 @@ static int encode8_hr(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 		cbits_out[1] = 0;
 		cbits_out[2] = 0;
 		cbits_out[3] = 1;
-		cbits_out[4] = 1;
+		cbits_out[4] = 0;
 	} else {
 		cbits_out[0] = 0;
 		cbits_out[1] = 0;
@@ -855,7 +874,7 @@ static int encode8_hr(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	/* XC1 .. XC2 */
 	memcpy(trau_bits + 1 * 8 + 6, fr->xc_bits, 2);
 	/* XC3 .. XC6 */
-	memcpy(trau_bits + 2 * 8 + 2, fr->xc_bits, 4);
+	memcpy(trau_bits + 2 * 8 + 2, fr->xc_bits + 2, 4);
 	/* D1 ..  D2 */
 	memcpy(trau_bits + 2 * 8 + 6, fr->d_bits, 2);
 	d_idx += 2;
@@ -868,12 +887,14 @@ static int encode8_hr(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	};
 
 	/* CRC0 .. CRC2 */
-	trau_bits[82] = fr->crc_bits[2];
-	trau_bits[83] = fr->crc_bits[1];
-	trau_bits[84] = fr->crc_bits[0];
+	memcpy(trau_bits + 73, fr->crc_bits, 3);
+
+	/* D45 .. D48 */
+	memcpy(trau_bits + 76, fr->d_bits + d_idx, 4);
+	d_idx += 4;
 
 	/* D49 .. D111 */
-	for (i = 10; i < 10 + 10; i++) {
+	for (i = 10; i < 10 + 9; i++) {
 		int offset = i * 8;
 		memcpy(trau_bits + offset + 1, fr->d_bits + d_idx, 7);
 		d_idx += 7;
@@ -1109,12 +1130,13 @@ static int decode8_data(struct osmo_trau_frame *fr, const ubit_t *trau_bits,
 	d_idx += 2;
 	/* D3 .. D8 */
 	memcpy(fr->d_bits + d_idx, trau_bits + 2 * 8 + 2, 6);
-	/* D9 .. D57 + D'1 .. D'57 */
-	for (i = 3; i < 20; i++) {
+	d_idx += 6;
+	/* D9 .. D63 + D'1 .. D'57 */
+	for (i = 3; i < 19; i++) {
 		memcpy(fr->d_bits + d_idx, trau_bits + i * 8 + 1, 7);
 		d_idx += 7;
 	}
-	/* D'58 .. D'62 */
+	/* D'58 .. D'63 */
 	memcpy(fr->d_bits + d_idx, trau_bits + 19 * 8 + 1, 6);
 	d_idx += 6;
 
@@ -1131,7 +1153,7 @@ static int encode8_data(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	trau_bits[1 * 8] = 1;
 	trau_bits[2 * 8] = 0;
 	trau_bits[2 * 8 + 1] = 1;
-	for (i = 3; i < 19; i++)
+	for (i = 3; i < 20; i++)
 		trau_bits[i * 8] = 1;
 	trau_bits[19 * 8 + 7] = 1;
 
@@ -1142,12 +1164,13 @@ static int encode8_data(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	d_idx += 2;
 	/* D3 .. D8 */
 	memcpy(trau_bits + 2 * 8 + 2, fr->d_bits + d_idx, 6);
-	/* D9 .. D57 + D'1 .. D'57 */
-	for (i = 3; i < 20; i++) {
+	d_idx += 6;
+	/* D9 .. D63 + D'1 .. D'57 */
+	for (i = 3; i < 19; i++) {
 		memcpy(trau_bits + i * 8 + 1, fr->d_bits + d_idx, 7);
 		d_idx += 7;
 	}
-	/* D'58 .. D'62 */
+	/* D'58 .. D'63 */
 	memcpy(trau_bits + 19 * 8 + 1, fr->d_bits + d_idx, 6);
 	d_idx += 6;
 
@@ -1278,6 +1301,7 @@ int osmo_trau_frame_encode(ubit_t *bits, size_t n_bits, const struct osmo_trau_f
 	case OSMO_TRAU16_FT_DATA:
 		return encode16_data(bits, fr);
 	case OSMO_TRAU16_FT_D145_SYNC:
+		return encode16_d145_sync(bits, fr);
 	case OSMO_TRAU16_FT_EDATA:
 		return encode16_edata(bits, fr);
 	case OSMO_TRAU8_SPEECH:
@@ -1348,7 +1372,7 @@ int osmo_trau_frame_decode_16k(struct osmo_trau_frame *fr, const ubit_t *bits,
 		return decode16_edata(fr, bits, dir);
 	case TRAU_FT_D145_SYNC:
 		fr->type = OSMO_TRAU16_FT_D145_SYNC;
-		return decode16_edata(fr, bits, dir);
+		return decode16_data(fr, bits, dir);
 
 	default:
 		return -EINVAL;
@@ -1359,7 +1383,7 @@ int osmo_trau_frame_decode_16k(struct osmo_trau_frame *fr, const ubit_t *bits,
 #define TRAU8_FT_AMR_NO_SPEECH_CMR	0x14	/* 1, 0, 1, 0, 0 */
 #define TRAU8_FT_AMR_475_515_590	0..7
 
-const uint8_t bit8_0[16] = { 0,  };
+static const uint8_t bit8_0[8] = { 0,  };
 
 /*!< check sync pattern for hr/data/oam */
 static bool is_hr(const ubit_t *bits)
@@ -1373,7 +1397,7 @@ static bool is_hr(const ubit_t *bits)
 		return false;
 	if (bits[16] != 0 || bits[17] != 1)
 		return false;
-	for (i = 24; i < 20 * 8; i += 16) {
+	for (i = 24; i < 20 * 8; i += 8) {
 		if (bits[i] != 1)
 			return false;
 	}
@@ -1395,7 +1419,7 @@ static bool is_amr_low(const ubit_t *bits)
 		return false;
 	if (bits[24] != 0 || bits[25] != 1)
 		return false;
-	for (i = 32; i < 20 * 8; i += 16) {
+	for (i = 32; i < 20 * 8; i += 8) {
 		if (bits[i] != 1)
 			return false;
 	}
@@ -1463,7 +1487,7 @@ int osmo_trau_frame_decode_8k(struct osmo_trau_frame *fr, const ubit_t *bits,
 				return decode8_hr(fr, bits, dir);
 			case 0x07:
 				return decode8_data(fr, bits, dir);
-			case 0x13:
+			case 0x0B:
 				return decode8_oam(fr, bits, dir);
 			}
 		} else {
